@@ -46,16 +46,17 @@ class Instrument:
     DEFAULT_TERMINATOR_WRITE = '\x0A'  # '\n'
     DEFAULT_TERMINATOR_READ = '\x0A'  # '\n'
     DEFAULT_TERMINATOR_RESET = '\x03'  # Ctrl-C
+    DEFAULT_BAUDRATE = 9600
     DEFAULT_TIMEOUT = 5
     DEFAULT_PROMPTS = re_compile( '[=#?!]>' )
     DEFAULT_PROMPTS_MULTILINE = re_compile( '[=#?!]?>' )
-    DEFAULT_LINE_LIMIT = 128
     DEFAULT_IDN = re_compile( '.*' )
 
     DEFAULT_TERMINAL_RESET = True
     DEFAULT_CHECK_IDN = True
     DEFAULT_READ_UNTIL_PROMPTS = True
     DEFAULT_CHECK_RETURN_COMMAND = True
+    DEFAULT_LINE_PROMPT_MODE = False
    
 
     def __init__( self, *args, **kwargs ):
@@ -72,7 +73,7 @@ class Instrument:
         elif self.warning:
             level = logging.WARNING
         else:
-            level = logging.FATAL
+            level = logging.ERROR
         logging.basicConfig( level=level, format=Env.LOG_FORMAT, datefmt=Env.LOG_DATEFMT )
         self.logger = logging.getLogger( self.DEFAULT_NAME )
         self.returned_cmd = None
@@ -87,7 +88,10 @@ class Instrument:
         if not 'port' in kwargs:
             kwargs['port'] = Env.PORT
         if not 'baudrate' in kwargs:
-            kwargs['baudrate'] = Env.BAUDRATE
+            if Env.BAUDRATE:
+                kwargs['baudrate'] = Env.BAUDRATE
+            else:
+                kwargs['baudrate'] = self.DEFAULT_BAUDRATE
         if not 'rtscts' in kwargs:
             kwargs['rtscts'] = Env.RTSCTS
         if not 'parity' in kwargs:
@@ -103,7 +107,7 @@ class Instrument:
         if 'check_idn' in kwargs:
             self.DEFAULT_CHECK_IDN = bool(kwargs['check_idn'])
         # some attributes 'connect', ...  need to be renamed for method conflict
-        for n in ['connect', 'timeout']:
+        for n in ['connect', 'baudrate', 'timeout']:
             kwargs['_'+n] = kwargs.pop(n)
         # attached as attributes
         for k, v in kwargs.items():
@@ -184,6 +188,12 @@ class Instrument:
   
     def readUntilPrompts( self, line_callback=None ):
         '''read until prompts'''
+        if self.DEFAULT_LINE_PROMPT_MODE:
+            return self._readUntilPromptsByLine( line_callback )
+        else:
+            return self._readUntilPromptsByChar( line_callback )
+
+    def _readUntilPromptsByChar( self, line_callback=None ):
         contents, newline_lst, newline_str = [], [], ''
         while True:
             byte = self.port.read(1)
@@ -195,7 +205,6 @@ class Instrument:
                     contents.append( newline_str )
                     self.logger.debug( '[R] '+ newline_str )
                     if line_callback is not None:
-                        # use this carefully
                         line_callback( newline_str )
                     newline_lst, newline_str = [], ''
                 else:
@@ -208,38 +217,74 @@ class Instrument:
                     raise CommandTimeoutError( ' | '.join(contents) )
                 else:
                     raise CommandTimeoutError( 'No response' )
-            #print( newline_str )  # for port debug
             match = self.prompts.match( newline_str )
             if match:
                 contents.append( newline_str )
                 #self.logger.debug( '[P] '+ newline_str )
                 return contents
 
-    def readLine( self, eol='\n', timeout=None, decode_utf8=True ):
+    def _readUntilPromptsByLine( self, line_callback=None ):
+        contents = []
+        while True:
+            newline_str = self.readLine( strip=False )
+            if newline_str is None:
+                if contents:
+                    raise Instrument.CommandTimeoutError( ' | '.join(contents) )
+                else:
+                    raise Instrument.CommandTimeoutError( 'No response' )
+            match = self.prompts.match( newline_str )
+            if match:
+                contents.append( newline_str )
+                #self.logger.debug( '[P] '+ newline_str )
+                return contents
+            else:
+                contents.append( newline_str )
+                self.logger.debug( '[R] '+ newline_str )
+                if line_callback is not None:
+                    line_callback( newline_str )
+                
+    def readLine( self, eol=None, timeout=None, decode_utf8=True, strip=True ):
         chars = []
-        t0 = time.time()
+        if eol is None:
+            eol = self.DEFAULT_TERMINATOR_READ
         if Env.PYTHON_V3:
             eol = bytes(eol, encoding='utf8')
+        if timeout is not None:
+            old_timeout = self.port.timeout
+            self.port.timeout = timeout
         while True:
             char = self.port.read(1) 
             if char:
                 if char == eol:
                     break
                 chars.append( char )
-            elif timeout:
-                if time.time() > t0 + timeout:
-                    break
-        ret = Env.EMPTY_BYTE.join(chars).rstrip()
-        if decode_utf8:
-            ret = ret.decode('utf8')
-            self.logger.debug( '[R] '+ ret )
+            else:
+                if chars:
+                    ret = Env.EMPTY_BYTE.join(chars)
+                    self.logger.debug( '[E] '+ str(ret) )
+                return None
+                #raise CommandTimeoutError( 'readLine timeout' )
+                #break
+        if timeout is not None:
+            self.port.timeout = old_timeout
+        if chars:
+            ret = Env.EMPTY_BYTE.join(chars)
+            if strip:
+                ret = ret.strip()
+            if decode_utf8:
+                ret = ret.decode('utf8')
+                self.logger.debug( '[R] '+ ret )
+            else:
+                self.logger.debug( '[R] '+ str(ret) )
+            return ret
         else:
-            self.logger.debug( '[R] '+ str(ret) )
-        return ret
+            return ''
 
     def writeLine( self, dat, encode_utf8=True ):
-        self.assertIsOpen() 
-        self.logger.debug( '[T] '+dat )
+        if isinstance( dat, bytes ):
+            self.logger.debug( '[T] '+str(dat) )
+        else:
+            self.logger.debug( '[T] '+dat )
         if encode_utf8:
             if Env.PYTHON_V3:
                 if isinstance( dat, str ):
@@ -251,18 +296,19 @@ class Instrument:
         self.port.write( self.DEFAULT_TERMINATOR_WRITE )
         self.port.flush()
    
-    def writeCommand( self, cmd ):
+    def writeCommand( self, cmd, strip=True ):
         '''write command and wait for prompts'''
-        cmd = cmd.strip()
+        if strip:
+            cmd = cmd.strip()
         self.writeLine( cmd )
         if self.DEFAULT_READ_UNTIL_PROMPTS:
             ret = self.readUntilPrompts()
-            #for line in [i.strip() for i in ret]:
-            #    self.logger.debug( '[R] '+ line )
             self.checkReturnedPrompt( ret )
             if cmd and self.DEFAULT_CHECK_RETURN_COMMAND and not Env.NO_ECHO_CHECK:
-                self.checkReturnCommand( ret, cmd )
-            return ret[1:-1] 
+                self.checkReturnedCommand( ret, cmd )
+                return ret[1:-1] 
+            else:
+                return ret[:-1] 
         else:
             return []
     
@@ -280,10 +326,10 @@ class Instrument:
                     print( e )
         return self.writeCommand( cmd )
   
-    def checkReturnCommand( self, ret, cmd ):
+    def checkReturnedCommand( self, ret, cmd ):
         '''assert command returned is valid'''
         cmdret = ret[0]
-        if Env.PYTHON_V3 and isinstance(cmd, bytes):
+        if Env.PYTHON_V3 and isinstance(cmdret, bytes):
             cmdret = cmdret.encode('utf8')
         if cmd != cmdret:
             raise ResponseError('Command %s, but returned %s'% (cmd, cmdret))
@@ -326,7 +372,7 @@ class Instrument:
         self.idn = ret[0].strip()
         if len(ret)>1:
             self.serial_number = ret[1].strip()
-        self.logger.info( 'IDN:%s', str(self.idn) )
+        self.logger.debug( 'IDN:%s', str(self.idn) )
         if check and (not Env.NO_IDN_CHECK):
             if not self.DEFAULT_IDN.match( self.idn ):
                 raise IDNMatchError(self.idn.split(',')[0])
@@ -409,9 +455,21 @@ class Port(object):
     def readall( self ):
         raise
 
+    def update_baudrate( self, baudrate ):
+        pass
+ 
     def update_timeout( self, timeout ):
         pass 
 
+    @property
+    def baudrate( self ):
+        return self._baudrate
+
+    @baudrate.setter
+    def baudrate( self, val ):
+        self._baudrate = val
+        self.update_baudrate( val )
+
     @property
     def timeout( self ):
         return self._timeout
@@ -444,23 +502,22 @@ class SerialPort(Port):
         try:
             self.ser.open()
             self._connected = True
+            self.ser.reset_output_buffer()
+            self.ser.reset_input_buffer()
         except IOError:
             raise PortNotFound( self.port )
         except Exception as e:
             #print( e )
             raise UnknownPortError( e )
-        if self._connected:
-            try:
-                self.ser.reset_input_buffer()
-                self.ser.reset_output_buffer()
-            except:
-                pass
 
     def disconnect( self ):
         if self._connected:
             self.ser.close()
             self._connected = False
  
+    def update_baudrate( self, baudrate ):
+        self.ser.baudrate = baudrate
+
     def update_timeout( self, timeout ):
         self.ser.timeout = timeout
         
