@@ -81,12 +81,14 @@ class ShellLab(Mcush.Mcush):
     daqDone = daq_done
     daqRead = daq_read
 
-    def measure( self, cmd, index=None, value=None ):
+    def measure( self, cmd, index=None, value=None, fvalue=None ):
         cmd = 'measure -c %s'% cmd
         if index is not None:
             cmd += ' -i %d'% index
         if value is not None:
             cmd += ' -v %d'% value
+        if fvalue is not None:
+            cmd += ' -V %f'% fvalue
         return self.writeCommand( cmd )
 
     def measure_start( self ):
@@ -124,7 +126,7 @@ class ShellLabLamp(Mcush.Mcush):
     DEFAULT_NAME = 'ShellLabLamp'
     DEFAULT_IDN = re_compile( 'ShellLab-L1[a-zA-Z]*,([0-9]+\.[0-9]+.*)' )
 
-    def lamp( self, color=None, red=None, green=None, blue=None, freq=None ):
+    def lamp( self, color=None, red=None, green=None, blue=None, freq=None, mode=None, count=None ):
         cmd = 'lamp'
         if color is not None:
             cmd += ' -c 0x%X'% color
@@ -136,6 +138,10 @@ class ShellLabLamp(Mcush.Mcush):
             cmd += ' -b %d'% blue
         if freq is not None:
             cmd += ' -f %s'% freq
+        if mode is not None:
+            cmd += ' -m %d'% mode
+        if count is not None:
+            cmd += ' -C %d'% count
         return self.writeCommand(cmd)
 
     def alarm( self, count=None, freq=None ):
@@ -146,16 +152,28 @@ class ShellLabLamp(Mcush.Mcush):
             cmd += ' -f %d'% freq
         return self.writeCommand(cmd)
  
-    def reset( self, lamp_freq=1, alarm_freq=1 ):
-        self.lamp( 0, freq=lamp_freq )
+    def reset( self, lamp_freq=1, alarm_freq=1, mode=None ):
+        self.lamp( 0, freq=lamp_freq, mode=mode )
         self.alarm( 0, freq=alarm_freq )
 
-    def color( self, c, freq=None ):
+    def mode( self, mode ):
+        if isinstance(mode, str):
+            if mode in ['blink', 'reset']:
+                mode = 0
+            elif mode in ['clock', 'clockwise', 'rotate']:
+                mode = 1
+            elif mode in ['anti', 'anticlockwise']:
+                mode = 2
+            else:
+                raise Exception('Unknown mode name %s'% mode)
+        self.lamp(mode=mode)
+
+    def color( self, c, freq=None, count=None ):
         if isinstance(c, str):
             if not c in COLOR_TAB:
                 raise Exception('Unknown color name %s'% c)
             c = COLOR_TAB[c]
-        self.lamp( c, freq=freq ) 
+        self.lamp( c, freq=freq, count=count ) 
 
 
 class ShellLabStrap(Mcush.Mcush):
@@ -172,7 +190,7 @@ class ShellLabStrap(Mcush.Mcush):
         cmd = 'strap -l%d'% length
         self.writeCommand(cmd)
 
-    def strap( self, color=None, red=None, green=None, blue=None, freq=None ):
+    def strap( self, color=None, red=None, green=None, blue=None, freq=None, count=None ):
         cmd = 'strap'
         if color is not None:
             cmd += ' -c 0x%X'% color
@@ -184,17 +202,19 @@ class ShellLabStrap(Mcush.Mcush):
             cmd += ' -b %d'% blue
         if freq is not None:
             cmd += ' -f %s'% freq
+        if count is not None:
+            cmd += ' -C %d'% count
         return self.writeCommand(cmd)
 
     def reset( self, freq=1 ):
         self.strap( 0, freq=freq )
 
-    def color( self, c, freq=None ):
+    def color( self, c, freq=None, count=None ):
         if isinstance(c, str):
             if not c in COLOR_TAB:
                 raise Exception('Unknown color name %s'% c)
             c = COLOR_TAB[c]
-        self.strap( c, freq=freq ) 
+        self.strap( c, freq=freq, count=count ) 
 
     def beep( self, freq=None, duration=0.05, times=1 ):
         # not supported
@@ -532,3 +552,220 @@ class ShellLabCANopen(ShellLabCAN):
 
 
 
+class ShellLabStepperMotor(Mcush.Mcush):
+    DEFAULT_NAME = 'ShellLabStepperMotor'
+    DEFAULT_IDN = re_compile( 'ShellLab(-[A-Z][0-9a-zA-Z\-]*)?,([0-9]+\.[0-9]+.*)' )
+
+    def motor( self, cmd, index=None, value=None ):
+        cmd = 'M -c%s'% cmd
+        if index is not None:
+            cmd += ' -i%d'% index
+        if value is not None:
+            cmd += ' -v%s'% value
+        return self.writeCommand( cmd )
+
+    def start( self, index=None ):
+        self.motor( 'start', index=index )
+
+    def stop( self, index=None ):
+        self.motor( 'stop', index=index )
+
+    def isStopped( self, index=None ):
+        return bool(int(self.motor('stopped', index=index)[0]))
+
+    def isStable( self, index=None ):
+        return bool(int(self.motor('stable', index=index)[0]))
+
+    def setPPR( self, ppr, index=None ):
+        self.motor( 'ppr', index=index, value=ppr )
+
+    def setSpeed( self, speed, index=None ):
+        self.motor( 'speed', index=index, value=speed )
+
+    def setDir( self, direction, index=None ):
+        self.motor( 'dir', index=index, value=direction )
+
+    def setFree( self, free, index=None ):
+        self.motor( 'free', index=index, value=free )
+
+    def waitForStable( self, index=None ):
+        while True:
+            if self.isStable(index=index):
+                break
+            time.sleep(0.1)
+
+    def waitForStopped( self, index=None ):
+        while True:
+            if self.isStopped(index=index):
+                break
+            time.sleep(0.1)
+
+
+class ShellLabSegmentDisplay(Mcush.Mcush):
+    DEFAULT_NAME = 'ShellLabSegmentDisplay'
+    DEFAULT_IDN = re_compile( 'ShellLab-L3[a-zA-Z]*,([0-9]+\.[0-9]+.*)' )
+
+    def __init__( self, *args, **kwargs ):
+        Instrument.Instrument.__init__( self, *args, **kwargs )
+        self.digits = int(self.disp('digits')[0])
+
+    def disp( self, cmd=None, idx=None, val=None, msg=None ):
+        command = 'd'
+        if cmd is not None:
+            command += ' -c%s'% cmd
+        if idx is not None:
+            command += ' -i%d'% idx
+        if val is not None:
+            command += ' -v%d'% val
+        if msg is not None:
+            command += ' -m"%s"'% msg
+        return self.writeCommand( command ) 
+
+    def brightness(self, new_brightness=7):
+        self.disp('brightness', val=new_brightness)
+
+    def clrscr(self):
+        self.disp('clrscr')
+
+    def blink(self, on, idx=None):
+        self.disp('blink', idx=idx, val=on)
+
+    def raw(self, val, idx):
+        self.disp('raw', idx=idx, val=val)
+   
+    def hex(self, val, idx):
+        self.disp('hex', idx=idx, val=val)
+
+    def ascii(self, val, idx):
+        self.disp('ascii', idx=idx, val=val)
+
+    def int(self, val, idx=None):
+        self.disp('int', idx=idx, val=val)
+ 
+    def msg(self, msg, idx=None):
+        self.disp(idx=idx, msg=msg)
+
+    def dispString(self, string, align='left'):
+        if align == 'right':
+            msg_len = len(string.replace('.','').replace(':',''))
+            self.msg( string, idx=self.digits-msg_len )
+        else:
+            self.msg( string )
+
+    def dispInteger(self, val, align='right'):
+        self.dispString( '%d'% val, align=align )
+
+    def dispHex(self, val, align='right'):
+        self.dispString( '%X'% val, align=align )
+
+    def dispFloat(self, val, precesion=1, align='right'):
+        fmt = '%%.%df'% precesion
+        output_str = fmt% val
+        if align == 'right':
+            output_len = len(output_str)-1
+            if output_len <= self.digits:
+                self.dispString( output_str, align='right' )
+            else:
+                self.dispString( output_str, align='left' )
+        else:
+            self.dispString( output_str, align='left' )
+
+
+class ShellLabSensor(Mcush.Mcush):
+    DEFAULT_NAME = 'ShellLab-Sensor'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-[A-Z][0-9a-zA-Z]*,([0-9]+\.[0-9]+.*)' )
+
+    def measure( self, cmd=None, idx=None, val=None ):
+        command = 'm'
+        if cmd is not None:
+            command += ' -c%s'% cmd
+        if idx is not None:
+            command += ' -i%d'% idx
+        if val is not None:
+            command += ' -v%d'% val
+        return self.writeCommand( command ) 
+
+ 
+class ShellLabSensorTH(ShellLabSensor):
+    '''SHT30 based temperature/humidity sensor'''
+    DEFAULT_NAME = 'ShellLab-Sensor-TH'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-TH,([0-9]+\.[0-9]+.*)' )
+
+    def configMPS( self, mps=0.5 ):
+        if mps >= 10.0:
+            val = 4
+        elif mps > 4.0:
+            val = 3
+        elif mps > 2.0:
+            val = 2
+        elif mps > 1.0:
+            val = 1
+        else:
+            val = 0
+        self.measure( 'mps', val=val )
+    
+    def configRepeatability( self, repeatability='high' ):
+        if repeatability == 'low':
+            val = 2
+        elif repeatability == 'medium':
+            val = 1
+        else:
+            val = 0
+        self.measure( 'repeatability', val=val )
+
+    def getTH( self ):
+        ret = self.measure()
+        a, b = ret[0].split(' ')
+        k1,v1 = a.split(':')
+        k2,v2 = b.split(':')
+        if k1 == 'T' and k2 == 'H':
+            return (float(v1), float(v2))
+        return None 
+
+
+class ShellLabSensorEncoder(ShellLabSensor):
+    '''Rotary encoder'''
+    DEFAULT_NAME = 'ShellLab-Sensor-Encoder'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-Encoder,([0-9]+\.[0-9]+.*)' )
+
+    def setPos( self, pos ):
+        self.measure('pos', val=pos)
+
+    def getPos( self ):
+        return float(self.measure('pos')[0])
+
+    def getSpeed( self ):
+        return float(self.measure('speed')[0])
+
+    def getVal( self ):
+        pos, val = self.measure()[0].split()
+        return float(pos), float(val)
+
+
+class ShellLabSensorPH(ShellLabSensor):
+    '''Acid/alkali PH sensor'''
+    DEFAULT_NAME = 'ShellLab-Sensor-PH'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-PH,([0-9]+\.[0-9]+.*)' )
+
+    def getPH( self ):
+        return None 
+
+
+class ShellLabSensorPressure(ShellLabSensor):
+    '''Pressure sensor'''
+    DEFAULT_NAME = 'ShellLab-Sensor-Pressure'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-Pressure,([0-9]+\.[0-9]+.*)' )
+
+    def getPressure( self ):
+        return None 
+
+
+class ShellLabSensorWeight(ShellLabSensor):
+    '''Weight sensor'''
+    DEFAULT_NAME = 'ShellLab-Sensor-Weight'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-Weight,([0-9]+\.[0-9]+.*)' )
+
+    def getWeight( self ):
+        return None 
+
+
