@@ -33,4 +33,11 @@ from . import Korad
 from . import Yaohua
 from . import OpenOCD
 from . import Dallas1W
+from . import Mlx90614
+from . import WitMotion
+from . import Sensirion
+from . import GCode
 from . import Tongmen  # require pymodbus
+from . import Zhouzheng  # require pymodbus
+from . import Miaoguan  # require pymodbus
+
